{-------------------------------------------------------------------------------
 TNBSDBListBox - Component to fill a listbox with a database field.

 This is a simple version that only fills the listbox. It is not aware
 of when a DB-Navigator scrolls the dataset.
 It's perfect to use when you just want the user to select a record
 from a database.

 property DataSource : Set this to the datasource. (just like TDBEdit and
                       the others)
 property DataField  : The fieldname.
 property ExtraField : Use this to store a value in the Items.Objects.
                       Really good if you want to maintain an index for
                       every item. The field must be a "value" field. It only
                       accepts: TIntegerField, TSmallintField, TWordField,
                       TBooleanField.
 property Current_Item : This string contains the current selected Item.
                         Easier to use than xxx.Items[xxx.ItemIndex];
 property Current_Value : The stored ExtraField for the currently selected item.
                          Easier to use than:
                          LongInt(xxx.Items.Objects[xxx.ItemIndex])

 TNBSDBListBox released as postcard-ware, which means that you are
 free to use (modify) this software and sourcecode as you see fit.
 But if you like it, and plan to use it, please send me a postcard
 with comments or suggestions.
 If you use this component in a commercial product, I would be greatful
 if you could give me some credits.


 Made by Andreas Oestlund, 1997.
 Address:  Andreas Ostlund
           Norrbro 39
           811 70 Jarbo
           Sweden

 E-Mail: andres@Kuai.se
 WWW: http://www.kuai.se/~andres

 Remember! Information wants to be free!
-------------------------------------------------------------------------------}

Unit NBSDBLst;

{$S-,W-,R-}

interface

Uses
    SysUtils, WinTypes, WinProcs, Messages, Classes, Controls, Forms,
    Graphics, StdCtrls, DB, DBTables;

Type
    TNBSDBListBox = Class(TCustomListBox)
    Private
       FDataLink : TFieldDataLink;
       FFillingItems : Boolean;
       FExtraField : String;

       Function GetDataSource : TDatasource;
       Procedure SetDataSource(Value : TDatasource);
       Function GetdataField  : String;
       procedure SetDataField(Value : String);
       Procedure SetExtraField(Value : String);
       Procedure CheckExtraField;
       Function GetCurrItem : String;
       Function GetcurrValue : LongInt;

       Procedure DataChange(Sender : TObject);
       Procedure ActiveChange(Sender : TObject);
       Procedure FillItems;
    Public
       Constructor Create(AOwner : TComponent); Override;
       Destructor Destroy; Override;

       property Items;
       property ItemIndex;

       property Current_Value : LongInt read GetCurrValue;
       property Current_Item : String read GetCurrItem;
    published
       property DataSource : TDataSource read GetDataSource Write SetDataSource;
       property DataField : String Read GetDataField Write SetDataField;
       property ExtraField : String read FExtraField Write SetExtraField;

       property Align;
       property BorderStyle;
       property Color;
       property Columns;
       property Ctl3D;
       property DragCursor;
       property DragMode;
       property Enabled;
       property ExtendedSelect;
       property Font;
       property IntegralHeight;
       property ItemHeight;
       property MultiSelect;
       property ParentColor;
       property ParentCtl3D;
       property ParentFont;
       property ParentShowHint;
       property PopupMenu;
       property ShowHint;
       property Sorted;
       property Style;
       property TabOrder;
       property TabStop;
       property Visible;
       property OnClick;
       property OnDblClick;
       property OnDragDrop;
       property OnDragOver;
       property OnDrawItem;
       property OnEndDrag;
       property OnEnter;
       property OnExit;
       property OnKeyDown;
       property OnKeyPress;
       property OnKeyUp;
       property OnMeasureItem;
       property OnMouseDown;
       property OnMouseMove;
       property OnMouseUp;
    End;

procedure Register;

implementation

Uses
    TypInfo,DsgnIntf;

Constructor TNBSDBListBox.Create(AOwner : TComponent);
     Begin
          inherited Create(AOwner);

          FDataLink := TFielddataLink.Create;
          FdataLink.Control := Self;
          FDataLink.OnDataChange := DataChange;
          FDataLink.OnActiveChange := ActiveChange;

          FFillingItems := False;
          FExtraField := '';
     End;

Destructor TNBSDBListBox.Destroy;
     Begin
          FDataLink.Free;

          inherited Destroy;
     End;

Procedure TNBSDBListBox.DataChange(Sender : TObject);
     begin
          If (FDataLink.DataSet = NIL) Or FFillingItems Or (FDataLink.FieldName = '') Or
             (Not FDataLink.DataSet.Active) Then Exit;

          FillItems;
     End;

Procedure TNBSDBListBox.CheckExtraField;
Var
   F : TField;

     Begin
          IF FExtraField = '' Then Exit;

          F := FDataLink.DataSet.FieldByName(FExtraField);
          If (F is TIntegerField) Or (F Is TSmallintField) Or (F Is TWordField) Or
             (F is TBooleanField) Then Exit;

            { If execution gets here, ExtraField is not acceptable }
          FExtraField := '';
     End;

Procedure TNBSDBListBox.ActiveChange(Sender : TObject);
     Begin
          If FFillingItems Then Exit;
          If Not FDataLink.Dataset.Active Then Items.Clear
           Else
               CheckExtraField;
     End;

Function TNBSDBListBox.GetDataSource : TDatasource;
     Begin
          Result := FDataLink.DataSource;
     End;

Procedure TNBSDBListBox.SetDataSource(Value : TDatasource);
     Begin
          FDataLink.DataSource := Value;
     End;

Function TNBSDBListBox.GetDataField  : String;
     Begin
          Result := FDataLink.FieldName;
     End;

procedure TNBSDBListBox.SetDataField(Value : String);
     Begin
          FDataLink.FieldName := Value;

          If (FDataLink.DataSet = NIL) Or FFillingItems Or
             (Not FDataLink.DataSet.Active) Then Exit;

          If Value = '' Then Items.Clear
           Else
               FillItems;
     end;

Procedure TNBSDBListBox.SetExtraField(Value : String);
     Begin
          FExtraField := Value;

          If FDataLink.DataSet = NIL Then Exit;

            { If DataSet is open, Check if the entered fieldtype is OK }
          If FDataLink.DataSet.Active Then CheckExtraField;
     end;

Function TNBSDBListBox.GetCurrItem : STring;
     Begin
          If ItemIndex = -1 Then
             Result := ''
           Else
               Result := Items[ItemIndex];
     End;

Function TNBSDBListBox.GetCurrValue : LongInt;
     Begin
          Result := -1;
          If (ItemIndex = -1) Or (FExtraField = '') Then Exit;

          Result := LongInt(Items.Objects[ItemIndex]);
     End;

Procedure TNBSDBListBox.FillItems;
Var
   DF,
   EF  : String;
   BookMark : TBookMark;
   NDX : LongInt;

     begin
          FFillingItems := True;

          With FDataLink Do
             Begin
                  DF := FDataLink.FieldName;
                  EF := FExtraField;

                  DataSet.DisableControls;
                  BookMark := DataSet.GetBookMark;

                  Items.Clear;
                  DataSet.First;
                  While Not DataSet.Eof Do
                      Begin
                           NDX := Items.Add(DataSet.FieldByName(DF).Text);

                           If (EF <> '') Then Items.Objects[NDX] := Pointer(DataSet.FieldByName(EF).AsInteger);

                           DataSet.Next;
                           If (csDesigning in ComponentState) And (NDX = 10) Then Break;
                      End;

                  DataSet.GotoBookMark(BookMark);
                  DataSet.FreeBookMark(BookMark);
                  DataSet.EnableControls;
             End;

          FFillingItems := False;
     End;

(*******************************************************************)
(** The following is cut from the VCL. It could have been so      **)
(** simple to just to override the TDataFieldProperty but Borland **)
(** did not define it in the interface section of DBREG.PAS.      **)
(** So here goes...                                               **)
Type
    TNBSDBListBoxPropEdit = class(TStringProperty)
    public
      Function GetDataSourcePropName: string; virtual;
      Function GetAttributes: TPropertyAttributes; override;
      Procedure GetValueList(List: TStrings); virtual;
      Procedure GetValues(Proc: TGetStrProc); override;
    end;

Function TNBSDBListBoxPropEdit.GetAttributes: TPropertyAttributes;
     begin
          Result := [paValueList, paSortList, paMultiSelect];
     end;

Procedure TNBSDBListBoxPropEdit.GetValues(Proc: TGetStrProc);
Var
  I: Integer;
  Values: TStringList;

     Begin
          Values := TStringList.Create;
          Try
             GetValueList(Values);
             For I := 0 To Values.Count - 1 Do Proc(Values[I]);
          Finally
             Values.Free;
          End;
     End;

Function TNBSDBListBoxPropEdit.GetDataSourcePropName : String;
     begin
          Result := 'DataSource';
     end;

Procedure TNBSDBListBoxPropEdit.GetValueList(List: TStrings);
Var
  Instance: TComponent;
  PropInfo: PPropInfo;
  DataSource: TDataSource;

     begin
          Instance := GetComponent(0);
          PropInfo := TypInfo.GetPropInfo(Instance.ClassInfo, GetDataSourcePropName);
          if (PropInfo <> nil) and (PropInfo^.PropType^.Kind = tkClass) then
           begin
                DataSource := TObject(GetOrdProp(Instance, PropInfo)) as TDataSource;
                if (DataSource <> nil) and (DataSource.DataSet <> nil) then
                DataSource.DataSet.GetFieldNames(List);
           end;
     end;


Procedure Register;
     begin
          RegisterComponents('NBS', [TNBSDBListBox]);
          RegisterPropertyEditor(TypeInfo(String), TNBSDBListBox, 'ExtraField', TNBSDBListBoxPropEdit);
     end;

end.
